;*************************************************
; FILE: PICKREC.SC
;
; Script to add record picking PROCs to a library
;
;        Copyright 1994 S/Wizardry, Ltd.
;              All Rights Reserved.
;
;  CONTAINS
;     Public PROCs:
;     ----------------
;     PickRec.AddArray
;     PickRec.DialogProc
;     PickRec.ResyncArray
;     PickRec.RefreshArray
;     PickRec.ResyncRecord
;     PickRec.RefreshRecord
;
;     Private PROCs
;     -------------
;     PickRec.InitProcs
;     PickRec.HandleOpen
;     PickRec.HandleClose
;     PickRec.HandleUpdate
;     PickRec.HandleArrive
;     PickRec.HandleDepart
;     PickRec.HandleSelect
;     PickRec.HandleAccept
;     PickRec.HandleCancel
;
;     PickRec.HandleKey
;     PickRec.HandleMouse
;     PickRec.HandleMessage
;     PickRec.HandleIdle
;
;     PickRec.InitKeyProcs
;     PickRec.HandleUp
;     PickRec.HandleDown
;     PickRec.HandlePgUp
;     PickRec.HandlePgDn
;     PickRec.HandleCtrlPgUp
;     PickRec.HandleCtrlPgDn
;     PickRec.HandleHome
;     PickRec.HandleEnd
;
;     PickRec.HandleBksp
;     PickRec.HandlePrintable
;
;     PickRec.HandleIns
;     PickRec.HandleDel
;     PickRec.HandleEnter
;
;     PickRec.ClearSearch
;     PickRec.Search
;*************************************************
; PROC: PickRec.AddArray
;
; PURPOSE: Validate and initialize control
;             dynarray
;          Load initial values into data array.
;
; PRE-CONDITION:  Table is on workspace
;                 Data array declared
;                 Control dynarray declared and
;                    contains user data
;                 The dialog need NOT be active.
;
; POST-CONDITION: Table is current
;                 Data array contains first list
;                    of elements
;                 Control dynarray validated and
;                    contains system data
;                 Search value cleared
;
; DESCRIPTION:
;    PickRec.Info is a dynarray.  Elements use by
;    AddArray include:
;       "Tag"          Tag of the controlled
;                      PICKARRAY
;       "Table"        Name of the table from
;                      which to pick a record
;       "Format Proc"  A PROC to format a record
;                      of the table into an
;                      element of PickRec.Array
;                      (see details in
;                      ResyncArray description)
;
; PLANS:
;    - Reorder table to specified index
;    - Load table, if necessary
;=================================================
PROC PickRec.AddArray(PickRec.Info, PickRec.Array)
PRIVATE TagValue
   ; Moderately lame sanity check
   IF NOT ISASSIGNED(PickRec.Magic) THEN
     PickRec.Magic = "xyzzy"
   ENDIF
   PickRec.Info["Magic"] = PickRec.Magic

   IF NOT PickRec.ValidInfo(PickRec.Info) THEN
     RETURN False
   ENDIF
   ; Move to table, order as specified, and get
   ; current record
   MOVETO PickRec.Info["Table"]
   IF ISASSIGNED(PickRec.Info["Index"])       AND
      ISASSIGNED(PickRec.Info["Index Field"]) THEN
     MOVETO FIELD PickRec.Info["Index Field"]
     ORDERTABLE SELECT PickRec.Info["Index"]
     HOME
   ENDIF
   PickRec.Info["RecNo"] = RECNO()
   ; Get first list of values
   PickRec.ResyncArray(PickRec.Info, PickRec.Array)

   ; Clear search string
   PickRec.ClearSearch()

   RETURN True
ENDPROC ; PickRec.AddArray
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.AddArray
  RELEASE PROCS PickRec.AddArray
ENDIF
;*************************************************
; PROC: PickRec.DialogProc
;
; PURPOSE:  Dispatch events for the controlled
;           PICKARRAY
;
; PRE-CONDITION:  PickRec.Info initialized by
;                    AddArray
;                 Table on workspace
;
; POST-CONDITION: Returns True or False depending
;                 on whether Paradox should also
;                 handle the event.
;
;*************************************************
PROC PickRec.DialogProc(TriggerType, TagValue,
                        EventValue, ElementValue,
                        PickRec.Info,
                        PickRec.Array)
   PRIVATE ProcIndex
   ; Raise Events to the same level as triggers
   ProcIndex = IIF(TriggerType = "EVENT",
                   EventValue["TYPE"],
                   TriggerType)

   ; Moderately lame sanity check
   IF NOT PickRec.ValidInfo(PickRec.Info) THEN
      QUIT "Control array bad"
   ENDIF

   ; Make sure we are supposed to handle this tag
   IF TagValue <> PickRec.Info["Tag"] THEN
     RETURN
   ENDIF

   ; Assume Paradox will also handle the event
   ; Some event handlers set this false.
   PickRec.Info["RetVal"] = True
   ; Initialized dispatch table
   IF NOT ISASSIGNED(PickRec.Procs) THEN
     PickRec.InitProcs()
   ENDIF
   ; Execute event handler, if any, for this event
   IF ISASSIGNED(PickRec.Procs[ProcIndex]) THEN
     EXECPROC(PickRec.Procs[ProcIndex])
   ENDIF

   RETURN PickRec.Info["RetVal"]
ENDPROC ; PickRec.DialogProc
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.DialogProc
  RELEASE PROCS PickRec.DialogProc
ENDIF
;*************************************************
; PROC: PickRec.ResyncArray
;
; PURPOSE: Resync contents of PickRec.Array with
;          table starting at current record of
;          table.
;
; PRE-CONDITION:  Table on workspace
;                 PickRec.Info["Format Proc"]
;                 names a defined proc with no
;                 arguments (see below).
;
; POST-CONDITION: On table
;                 PickRec.Array updated
;                 Record in table NOT changed
; PLANS:
;   - Should this do a ClearSearch()?
;
; DESCRIPTION:
;    The format proc takes the current record of
;    the current table and sets PickRec.Format to
;    what should be displayed to the user for that
;    record.  It might be as simple as:
;
;    PROC List.Format()
;       PickRec.Format =
;                    [Last Name]+","+[First Name]
;    ENDPROC
;
;    NOTE: The format proc is called via EXECPROC
;    and, therefore, cannot take any arguments.
;
;    NOTE: Since the space bar is used to select
;    items in a list, blanks cannot be entered in
;    the search string and should be avoided in
;    the formatted line.
;
;*************************************************
PROC PickRec.ResyncArray(PickRec.Info,
                         PickRec.Array)
PRIVATE I, HoldRec

   MOVETO PickRec.Info["Table"]
   ; Clear array (in case there aren't enough
   ; records to fill it)
   FOR I FROM 1 TO PickRec.Info["Elements"]
     PickRec.Array[I] = ""
   ENDFOR

   ; Fill the array
   HoldRec = RecNo()
   FOR I FROM 1 TO PickRec.Info["Elements"]
     EXECPROC PickRec.Info["Format Proc"]
     PickRec.Array[I] = PickRec.Format
     IF ATLAST() THEN
       QUITLOOP
     ENDIF
     DOWN
   ENDFOR
   MOVETO RECORD HoldRec

   PickRec.Info["Picked"] = 1

ENDPROC ; PickRec.ResyncArray
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.ResyncArray
  RELEASE PROCS PickRec.ResyncArray
ENDIF
;*************************************************
; PROC: PickRec.RefreshArray
;
; PURPOSE: Resync array with table and update
;          dialog
;
; PRE-CONDITION:  As for ResyncArray plus in
;                 dialog
;
; POST-CONDITION: Array and dialog updated
;
;*************************************************
PROC PickRec.RefreshArray(PickRec.Info,
                          PickRec.Array)

  PickRec.ResyncArray(PickRec.Info, PickRec.Array)
  PickRec.HandleUpdate()
  REFRESHCONTROL PickRec.Info["Tag"]

ENDPROC ; PickRec.RefreshArray
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.RefreshArray
  RELEASE PROCS PickRec.RefreshArray
ENDIF
;*************************************************
; PROC: PickRec.ResyncRecord
;
; PURPOSE:  Resynchronize an element of the array
;   with the current record in the table.
;
; PRE-CONDITION:  On table, Format Proc defined
;
; POST-CONDITION: Array element updated.
;
;*************************************************
PROC PickRec.ResyncRecord(PickRec.Info,
                          PickRec.Array)
   EXECPROC PickRec.Info["Format Proc"]
   PickRec.Array[PickRec.Info["Picked"]] =
                                    PickRec.Format

ENDPROC ; PickRec.ResyncRecord
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.ResyncRecord
  RELEASE PROCS PickRec.ResyncRecord
ENDIF
;*************************************************
; PROC: PickRec.RefreshRecord
;
; PURPOSE:  Update current element of array to
;           reflect changes in current record of
;           table.
;
; PRE-CONDITION:  Dialog active
;                 Table current
;
; POST-CONDITION: Array element updated
;
;*************************************************
PROC PickRec.RefreshRecord(PickRec.Info,
                           PickRec.Array)
   PickRec.ResyncRecord(PickRec.Info,
                        PickRec.Array)
   PickRec.HandleUpdate()
   REFRESHCONTROL PickRec.Info["Tag"]

ENDPROC ; PickRec.RefreshRecord
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.RefreshRecord
   RELEASE PROCS PickRec.RefreshRecord
ENDIF
;*************************************************
; PROC: PickRec.InitProcs
;
; PURPOSE:  Initialize dispatch table for event
;           handlers
;
; PRE-CONDITION:  None.
;
; POST-CONDITION: PickRec.Procs initialized.
;
;*************************************************
PROC PickRec.InitProcs()
   DYNARRAY PickRec.Procs[]
   ; Triggers
   PickRec.Procs["OPEN"] = "PickRec.HandleOpen"
   PickRec.Procs["CLOSE"] = "PickRec.HandleClose"
   PickRec.Procs["UPDATE"] = "PickRec.HandleUpdate"
   PickRec.Procs["ARRIVE"] = "PickRec.HandleArrive"
   PickRec.Procs["DEPART"] = "PickRec.HandleDepart"
   PickRec.Procs["SELECT"] = "PickRec.HandleSelect"
   PickRec.Procs["ACCEPT"] = "PickRec.HandleAccept"
   PickRec.Procs["CANCEL"] = "PickRec.HandleCancel"
   ; Events
   PickRec.Procs["KEY"] = "PickRec.HandleKey"
   PickRec.Procs["MOUSE"] = "PickRec.HandleMouse"
;   PickRec.Procs["MESSAGE"] =
;                         "PickRec.HandleMessage"
;   PickRec.Procs["IDLE"] = "PickRec.HandleIdle"

ENDPROC ; PickRec.InitProcs
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.InitProcs
  RELEASE PROCS PickRec.InitProcs
ENDIF
;*************************************************
; PROC: PickRec.HandleOpen
;
; PURPOSE:  Do update when dialog first displayed.
;
; PRE-CONDITION:  As for HandleUpdate
;
; POST-CONDITION: As for HandleUpdate
;
;*************************************************
PROC PickRec.HandleOpen()
  PickRec.HandleUpdate()
ENDPROC ; PickRec.HandleOpen
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleOpen
  RELEASE PROCS PickRec.HandleOpen
ENDIF
;
;*************************************************
; PROC: PickRec.HandleClose
;
; PURPOSE:  Stub
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleClose()
ENDPROC ; PickRec.HandleClose
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleClose
  RELEASE PROCS PickRec.HandleClose
ENDIF
;*************************************************
; PROC: PickRec.HandleUpdate
;
; PURPOSE:  Handle change in picked element.  Call
;    user-defined update proc, if defined.
;
; PRE-CONDITION:  On table
;
; POST-CONDITION: PickRec.Values has current record
;                 values.
;
; DESCRIPTION:
;    The update proc typically updates canvas
;    elements from values in the currently
;    selected record.  It might even lookup
;    details from another table like:
;
;    PROC List.Update()
;       PRIVATE Details
;       GETRECORD DetailTable PickRec.Values["ID"]
;          TO DYNARRAY Details
;       Canvas1 = Details["Field1"]
;       Canvas2 = Details["Field2"]
;       REPAINTDIALOG
;    ENDPROC
;
;    NOTE: The update proc is called via EXECPROC
;    and, therefore, cannot take any arguments.
;*************************************************
PROC PickRec.HandleUpdate()
   ; Save control data
   PickRec.Info["RecNo"] = RecNo()
   COPYTODYNARRAY PickRec.Values
   IF ISASSIGNED(PickRec.Info["Update Proc"]) THEN
     EXECPROC PickRec.Info["Update Proc"]
   ENDIF
ENDPROC ; PickRec.HandleUpdate
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleUpdate
  RELEASE PROCS PickRec.HandleUpdate
ENDIF
;*************************************************
; PROC: PickRec.HandleArrive
;
; PURPOSE:  Make sure table is as expected on
;    arrival on control
;
; PRE-CONDITION:  Table on workspace
;
; POST-CONDITION: Table is current, current
;     record selected
;*************************************************
PROC PickRec.HandleArrive()
   ; Get to the table for the control
   MOVETO PickRec.Info["Table"]

   ; Reset the index order
   IF ISASSIGNED(PickRec.Info["Index"]) AND
      ISASSIGNED(PickRec.Info["Index Field"]) THEN
     MOVETO FIELD PickRec.Info["Index Field"]
     ORDERTABLE SELECT PickRec.Info["Index"]
   ENDIF

   ; Move to correct record
   MOVETO RECORD PickRec.Info["RecNo"]
ENDPROC ; PickRec.HandleArrive
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleArrive
  RELEASE PROCS PickRec.HandleArrive
ENDIF
;
;*************************************************
; PROC: PickRec.HandleDepart
;
; PURPOSE:  Stub
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleDepart()
ENDPROC ; PickRec.HandleDepart
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleDepart
  RELEASE PROCS PickRec.HandleDepart
ENDIF
;*************************************************
; PROC: PickRec.HandleSelect
;
; PURPOSE:  Execute user-defined select proc,
;    if defined.
;
; PRE-CONDITION:  PickRec.Info["Select Proc"]
;                 names a proc with no arguments.
;
; POST-CONDITION: Select proc, if any, executed.
;
;*************************************************
PROC PickRec.HandleSelect()
  IF PickRec.Array[PickRec.Info["Picked"]]<>"" AND
     ISASSIGNED(PickRec.Info["Select Proc"])  THEN
    EXECPROC PickRec.Info["Select Proc"]
  ENDIF
ENDPROC   ; PickRec.HandleSelect
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleSelect
  RELEASE PROCS PickRec.HandleSelect
ENDIF
;*************************************************
; PROC: PickRec.HandleAccept
;
; PURPOSE:  Stub
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleAccept()
ENDPROC ; PickRec.HandleAccept
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleAccept
  RELEASE PROCS PickRec.HandleAccept
ENDIF
;
;*************************************************
; PROC: PickRec.HandleCancel
;
; PURPOSE:  Stub
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleCancel()
ENDPROC ; PickRec.HandleCancel
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleCancel
   RELEASE PROCS PickRec.HandleCancel
ENDIF
;*************************************************
; PROC: PickRec.HandleKey
;
; PURPOSE:  Dispatch key event
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleKey()
   PRIVATE ProcIndex

   ProcIndex = EventValue["KeyCode"]

   ; Initialize key event handler dispatch table,
   ; if necessary
   IF NOT ISASSIGNED(PickRec.KeyProcs) THEN
     PickRec.InitKeyProcs()
   ENDIF
   ; Dispatch key event handler
   IF ISASSIGNED(PickRec.KeyProcs[ProcIndex]) THEN
     EXECPROC(PickRec.KeyProcs[ProcIndex])
   ENDIF

ENDPROC ; PickRec.HandleKey
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleKey
   RELEASE PROCS PickRec.HandleKey
ENDIF
;*************************************************
; PROC: PickRec.InitKeyProcs
;
; PURPOSE:  Initialize key event handler dispatch
;    table
;
; PRE-CONDITION:  None
;
; POST-CONDITION: PickRec.KeyProcs initialized.
;
;*************************************************
PROC PickRec.InitKeyProcs()
   DYNARRAY PickRec.KeyProcs[]

   ; Navigation keys
   PickRec.KeyProcs[ASC("Up")] =
                          "PickRec.HandleUp"
   PickRec.KeyProcs[ASC("Down")] =
                          "PickRec.HandleDown"
   PickRec.KeyProcs[ASC("PgUp")] =
                          "PickRec.HandlePgUp"
   PickRec.KeyProcs[ASC("PgDn")] =
                          "PickRec.HandlePgDn"
   PickRec.KeyProcs[ASC("CtrlPgUp")] =
                          "PickRec.HandleCtrlPgUp"
   PickRec.KeyProcs[ASC("CtrlPgDn")] =
                          "PickRec.HandleCtrlPgDn"
   PickRec.KeyProcs[ASC("Home")] =
                          "PickRec.HandleHome"
   PickRec.KeyProcs[ASC("End")] =
                          "PickRec.HandleEnd"
   ; Keys which affect search value.
   PickRec.KeyProcs[ASC("Backspace")] =
                          "PickRec.HandleBksp"
   ; Printable characters (except 32, blank,
   ; because blank is used for select)
   FOR C FROM 33 TO 126
      PickRec.KeyProcs[C] =
                         "PickRec.HandlePrintable"
   ENDFOR
   ; Commonly used list manipulation keys
   PickRec.KeyProcs[ASC("Ins")] =
                        "PickRec.HandleIns"
   PickRec.KeyProcs[ASC("Del")] =
                        "PickRec.HandleDel"
   PickRec.KeyProcs[ASC("Enter")] =
                        "PickRec.HandleEnter"

ENDPROC ; PickRec.InitKeyProcs
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.InitKeyProcs
   RELEASE PROCS PickRec.InitKeyProcs
ENDIF
;*************************************************
; PROC: PickRec.HandleUp
;
; PURPOSE:  Handle up arrow by moving up one
;    record in table
;
; PRE-CONDITION:  On table, in dialog
;
; POST-CONDITION: Item changed, update handled,
;    array updated if necc.
;
;*************************************************
PROC PickRec.HandleUp()
   ; If at top of table, return
   IF ATFIRST() THEN
     PickRec.Info["RetVal"] = False
     RETURN
   ENDIF
   PickRec.ClearSearch()

   ; Move up one record in table
   UP
   ; If still in current list, update from new
   ; record and return to let Paradox move the
   ; cursor up in the PICKARRAY otherwise, shift
   ; the values down, refresh and update from
   ; the new record and keep Paradox from playing
   ; with the PICKARRAY
   IF PickRec.Info["Picked"] > 1 THEN
      PickRec.HandleUpdate()
   ELSE
      FOR I FROM PickRec.Info["Elements"]
            TO 2 STEP -1
         PickRec.Array[I] = PickRec.Array[I-1]
      ENDFOR
      PickRec.RefreshRecord(PickRec.Info,
                            PickRec.Array)
      PickRec.Info["RetVal"] = False
   ENDIF
ENDPROC ; PickRec.HandleUp
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleUp
   RELEASE PROCS PickRec.HandleUp
ENDIF
;*************************************************
; PROC: PickRec.HandleDown
;
; PURPOSE:  Handle down arrow by moving down one
;    record in table
;
; PRE-CONDITION:  On table, in dialog
;
; POST-CONDITION: Control updated
;
;*************************************************
PROC PickRec.HandleDown()
   ; If at bottom of table, return
   IF ATLAST() THEN
      PickRec.Info["RetVal"] = False
      RETURN
   ENDIF
   PickRec.ClearSearch()
   ; Move down one record
   DOWN

   ; If still in current list, update from new
   ; record and return to let Paradox move the
   ; cursor down in the PICKARRAY otherwise, shift
   ; values up, refresh and update from new
   ; record and keep Paradox from messing with the
   ; PICKARRAY
   IF PickRec.Info["Picked"] <
                    PickRec.Info["Elements"] THEN
     PickRec.HandleUpdate()
   ELSE
     FOR I FROM 1 TO PickRec.Info["Elements"] - 1
       PickRec.Array[I] = PickRec.Array[I+1]
     ENDFOR
     PickRec.RefreshRecord(PickRec.Info,
                           PickRec.Array)
     PickRec.Info["RetVal"] = False
   ENDIF
ENDPROC ; PickRec.HandleDown
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleDown
   RELEASE PROCS PickRec.HandleDown
ENDIF
;*************************************************
;  PickRec.HandlePgUp
;
; PURPOSE:  Handle PageUp key by moving up up to
;    PickRec.Info["Elements"] records in table
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array updated
;
;*************************************************
PROC PickRec.HandlePgUp()
   ; If at top of table, return
   IF ATFIRST() THEN
     PickRec.Info["RetVal"] = False
     RETURN
   ENDIF

   PickRec.ClearSearch()
   ; Move to top of current page
   FOR I FROM 1 TO PickRec.Info["Picked"] - 1
     UP
   ENDFOR

   ; Move up one page
   FOR I FROM 1 TO PickRec.Info["Elements"]
     IF ATFIRST() THEN
       QUITLOOP
     ENDIF
     UP
   ENDFOR
   ; Update array and dialog
   PickRec.RefreshArray(PickRec.Info,
                        PickRec.Array)
ENDPROC ; PickRec.HandlePgUp
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandlePgUp
   RELEASE PROCS PickRec.HandlePgUp
ENDIF
;*************************************************
; PROC: PickRec.HandlePgDn
;
; PURPOSE:  Handle PageDown key by moving down up
;    to PickRec.Info["Elements"] in table
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array updated
;
;*************************************************
PROC PickRec.HandlePgDn()
   PickRec.Info["RetVal"] = False
   ; If at bottom of table, return
   IF ATLAST() THEN
     RETURN
   ENDIF
   PickRec.ClearSearch()
   ; Move to last line on page
   FOR I FROM PickRec.Info["Picked"]
         TO PickRec.Info["Elements"] - 1
     IF ATLAST() THEN
       QUITLOOP
     ENDIF
     PickRec.Info["Picked"] =
                        PickRec.Info["Picked"] + 1
     DOWN
   ENDFOR
   ; If last line on page is last record, update
   ; dialog and return otherwise, move to first
   ; record of new page and update array and
   ; dialog
   IF ATLAST() THEN
      PickRec.HandleUpdate()
      REFRESHCONTROL PickRec.Info["Tag"]
   ELSE
      DOWN
      PickRec.RefreshArray(PickRec.Info,
                           PickRec.Array)
   ENDIF
ENDPROC ; PickRec.HandlePgDn
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandlePgDn
  RELEASE PROCS PickRec.HandlePgDn
ENDIF
;*************************************************
; PROC: PickRec.HandleCtrlPgUp
;
; PURPOSE:  Handle Ctrl-PageUp by moving to top of
;    table
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array updated
;
;*************************************************
PROC PickRec.HandleCtrlPgUp()
  PickRec.ClearSearch()
  HOME
  PickRec.RefreshArray(PickRec.Info,
                       PickRec.Array)
  PickRec.Info["RetVal"] = False
ENDPROC ; PickRec.HandleCtrlPgUp
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleCtrlPgUp
  RELEASE PROCS PickRec.HandleCtrlPgUp
ENDIF
;*************************************************
; PROC: PickRec.HandleCtrlPgDn
;
; PURPOSE:  Handle Ctrl-PageDown by moving to bottom of table
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array updated
;
;*************************************************
PROC PickRec.HandleCtrlPgDn()
PRIVATE HoldRec, I

  PickRec.ClearSearch()
  END
  HoldRec = RecNo()

  ; If moved off page, resync array
  IF RecNo() - PickRec.Info["RecNo"] >=
                    PickRec.Info["Elements"] THEN
     FOR I FROM 1 TO PickRec.Info["Elements"] - 1
       IF ATFIRST() THEN
         QUITLOOP
       ENDIF
       UP
     ENDFOR
     PickRec.ResyncArray(PickRec.Info,
                          PickRec.Array)
  ENDIF

  ; There may be less than a full page,
  ; so don't move to last element, move to
  ; last non-blank element
  PickRec.Info["Picked"] =
                          PickRec.Info["Elements"]
  WHILE PickRec.Info["Picked"] > 1 AND
        PickRec.Array[PickRec.Info["Picked"]] = ""
    PickRec.Info["Picked"] =
                        PickRec.Info["Picked"] - 1
  ENDWHILE
  MOVETO RECORD HoldRec
  PickRec.HandleUpdate()
  REFRESHCONTROL PickRec.Info["Tag"]
  PickRec.Info["RetVal"] = False
ENDPROC ; PickRec.HandleCtrlPgDn
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleCtrlPgDn
  RELEASE PROCS PickRec.HandleCtrlPgDn
ENDIF
;*************************************************
; PROC: PickRec.HandleHome
;
; PURPOSE:  Handle Home key by moving to first
;    visible record
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array updated
;
;*************************************************
PROC PickRec.HandleHome()
   ; If at top of table, return
   IF ATFIRST() THEN
     PickRec.Info["RetVal"] = False
     RETURN
   ENDIF

   PickRec.ClearSearch()
   ; Move to top of list
   FOR I FROM 1 TO PickRec.Info["Picked"] - 1
     UP
   ENDFOR
   PickRec.HandleUpdate()

   ; Let Paradox move the cursor to the
   ; first element

ENDPROC ; PickRec.HandleHome
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleHome
   RELEASE PROCS PickRec.HandleHome
ENDIF
;*************************************************
; PROC: PickRec.HandleEnd
;
; PURPOSE: Handle End key by moving to last
;    visible record
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleEnd()
   PRIVATE I

   PickRec.Info["RetVal"] = False

   ; If at bottom of table, return
   IF ATLAST() THEN
      RETURN
   ENDIF

   PickRec.ClearSearch()
   ; Move to bottom of list
   FOR I FROM PickRec.Info["Picked"]
         TO PickRec.Info["Elements"] - 1
     IF ATLAST() THEN
       QUITLOOP
     ENDIF
     DOWN
   ENDFOR

   PickRec.HandleUpdate()

   ; There may be less than a full page so don't
   ; move to the last element, move to the last
   ; non-blank element
   PickRec.Info["Picked"] =
                          PickRec.Info["Elements"]
   WHILE PickRec.Info["Picked"] > 1 AND
         PickRec.Array[PickRec.Info["Picked"]] = ""
     PickRec.Info["Picked"] =
                       PickRec.Info["Picked"] - 1
   ENDWHILE
   REFRESHCONTROL PickRec.Info["Tag"]
ENDPROC ; PickRec.HandleEnd
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleEnd
   RELEASE PROCS PickRec.HandleEnd
ENDIF
;*************************************************
; PROC: PickRec.HandlePrintable
;
; PURPOSE:  Handle printable character by updating
;    search value and searching if necessary
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array and control updated as
;    appropriate
;
;*************************************************
PROC PickRec.HandlePrintable()
PRIVATE  L, Prefix, PickRec.SearchValue
  IF NOT ISASSIGNED(PickRec.Info["Search Value"])
    THEN PickRec.Info["Search Value"] = ""
  ENDIF

  PickRec.SearchValue =
         PickRec.Info["Search Value"] +
         FORMAT("CU",CHR(EventValue["KEYCODE"]))

  IF PickRec.Search() THEN
    PickRec.Info["Search Value"] =
                              PickRec.SearchValue
    REPAINTDIALOG
  ENDIF
ENDPROC ; PickRec.HandlePrintable
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandlePrintable
   RELEASE PROCS PickRec.HandlePrintable
ENDIF
;*************************************************
; PROC: PickRec.HandleBksp
;
; PURPOSE:  Handle backspace by removing last
;    character from search value and searching
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array and control updated
;    as needed
;
;*************************************************
PROC PickRec.HandleBksp()
   PRIVATE L, PickRec.SearchValue
   IF NOT ISASSIGNED(PickRec.Info["Search Value"])
     THEN PickRec.Info["Search Value"] = ""
   ENDIF

   PickRec.SearchValue =
                      PickRec.Info["Search Value"]
   L = LEN(PickRec.SearchValue)
   IF L > 0 THEN
     PickRec.SearchValue =
               SUBSTR(PickRec.SearchValue, 1, L-1)
   ENDIF

   IF PickRec.Search() THEN
     PickRec.Info["Search Value"] =
                              PickRec.SearchValue
     REPAINTDIALOG
   ENDIF
ENDPROC ; PickRec.HandleBksp
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleBksp
   RELEASE PROCS PickRec.HandleBksp
ENDIF
;*************************************************
; PROC: PickRec.ClearSearch
;
; PURPOSE:  Clear search value.
;
; PRE-CONDITION:  None.
;
; POST-CONDITION: Search value cleared
;
;*************************************************
PROC PickRec.ClearSearch()
   PickRec.Info["Search Value"] = ""
   REPAINTDIALOG
ENDPROC ; PickRec.ClearSearch
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.ClearSearch
   RELEASE PROCS PickRec.ClearSearch
ENDIF
;*************************************************
; PROC: PickRec.Search
;
; PURPOSE:  Update array and control when search
;    value changes to simulate incremental search
;
; PRE-CONDITION:  On table and in dialog
;
; POST-CONDITION: Array and control updated as
;    needed.
;
;*************************************************
PROC PickRec.Search()
   PRIVATE FirstEl, LastElement

   ; Paradox should not do any searching
   PickRec.Info["RetVal"] = False

   ; If the new search value is outside of the
   ; array, call the user-provided search function
   FirstElement =
         FORMAT("CU", PickRec.Array[1])
   LastElement =
      FORMAT("CU",
         PickRec.Array[PickRec.Info["Elements"]])
   ; WUZ - This may test against a blank element
   ; if there is less than one page.
   IF PickRec.SearchValue > LastElement OR
      PickRec.SearchValue < FirstElement THEN
     IF ISASSIGNED(PickRec.Info["Search Proc"])
      THEN
         EXECPROC PickRec.Info["Search Proc"]
         IF PickRec.RetVal THEN
            PickRec.RefreshArray(PickRec.Info,
                                 PickRec.Array)
            RETURN True
         ENDIF
     ENDIF
     RETURN False
   ENDIF

   ; Search value should be in the array
   ; if it exists.

   L = LEN(PickRec.SearchValue)
   FOR I FROM 1 TO PickRec.Info["Picked"] - 1
      UP
   ENDFOR
   FOR I FROM 1 TO PickRec.Info["Elements"]
      Prefix = SUBSTR(PickRec.Array[I], 1, L)
      Prefix = FORMAT("CU", Prefix)
      IF Prefix = PickRec.SearchValue THEN
         PickRec.Info["Picked"] = I
         PickRec.HandleUpdate()
         REFRESHCONTROL TagValue
         RETURN True
      ENDIF
      DOWN
   ENDFOR
   MOVETO RECORD PickRec.Info["RecNo"]
   RETURN False
ENDPROC ; PickRec.Search
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.Search
   RELEASE PROCS PickRec.Search
ENDIF
;*************************************************
; PROC: PickRec.ValidInfo
;
; PURPOSE:  Validate PickRec.Info
;
; PRE-CONDITION:  None.
;
; POST-CONDITION: None.
;
;*************************************************
PROC PickRec.ValidInfo(PickRec.Info)
   ; PickRec.Info must be a dynarray
   IF TYPE(PickRec.Info) <> "DY" THEN
      RETURN False
   ENDIF

   ; Moderately lame sanity check
   IF NOT ISASSIGNED(PickRec.Info["Magic"])
   OR PickRec.Info["Magic"] <> PickRec.Magic THEN
      RETURN False
   ENDIF

   ; The tag must be specified
   IF NOT ISASSIGNED(PickRec.Info["Tag"]) THEN
      RETURN False
   ENDIF

   ; The table must be specified
   IF NOT ISASSIGNED(PickRec.Info["Table"])
   OR NOT ISTABLE(PickRec.Info["Table"]) THEN
      RETURN False
   ENDIF

   RETURN True
ENDPROC ; PickRec.ValidInfo
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.ValidInfo
   RELEASE PROCS PickRec.ValidInfo
ENDIF
;*************************************************
; PROC: PickRec.HandleIns
;
; PURPOSE:  Execute user-supplied PROC when INS
;    is pressed (presumably, to insert a new
;    record in the table).
;
; PRE-CONDITION:  As for Ins Proc
;
; POST-CONDITION: At for Ins Proc
;
;*************************************************
PROC PickRec.HandleIns()
   IF ISASSIGNED(PickRec.Info["Ins Proc"]) THEN
      EXECPROC PickRec.Info["Ins Proc"]
   ENDIF
ENDPROC ; PickRec.HandleIns
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleIns
   RELEASE PROCS PickRec.HandleIns
ENDIF
;
;*************************************************
; PROC: PickRec.HandleDel
;
; PURPOSE:  Execute user-supplied PROC when DEL
;    is pressed (presumably to delete a record
;    from the table).
;
; PRE-CONDITION:  As for Del Proc
;
; POST-CONDITION: As for Del Proc
;
;*************************************************
PROC PickRec.HandleDel()
   IF ISASSIGNED(PickRec.Info["Del Proc"]) THEN
      EXECPROC PickRec.Info["Del Proc"]
   ENDIF
ENDPROC ; PickRec.HandleDel
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleDel
   RELEASE PROCS PickRec.HandleDel
ENDIF
;*************************************************
; PROC: PickRec.HandleEnter
;
; PURPOSE:
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleEnter()
   IF ISASSIGNED(PickRec.Info["Enter Proc"]) THEN
      EXECPROC(PickRec.Info["Enter Proc"])
   ENDIF
ENDPROC ; PickRec.HandleEnter
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleEnter
   RELEASE PROCS PickRec.HandleEnter
ENDIF
;*************************************************
; PROC: PickRec.HandleMouse
;
; PURPOSE:  Reject mouse events in a PickRec
;    controlled PICKARRAY
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleMouse()
   PickRec.Info["RetVal"] = False
ENDPROC ; PickRec.HandleMouse
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleMouse
   RELEASE PROCS PickRec.HandleMouse
ENDIF
;
;*************************************************
; PROC: PickRec.HandleMessage
;
; PURPOSE:  Stub
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleMessage()
ENDPROC ; PickRec.HandleMessage
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleMessage
   RELEASE PROCS PickRec.HandleMessage
ENDIF
;
;*************************************************
; PROC: PickRec.HandleIdle
;
; PURPOSE: Stub
;
; PRE-CONDITION:
;
; POST-CONDITION:
;
;*************************************************
PROC PickRec.HandleIdle()
ENDPROC ; PickRec.HandleIdle
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleIdle
  RELEASE PROCS PickRec.HandleIdle
ENDIF
